@@ -128,14 +128,24 @@ class FuzzerPassDonateModules : public FuzzerPass {
       std::map<uint32_t, uint32_t>* original_id_to_donated_id,
       std::vector<protobufs::Instruction>* donated_instructions);
 
+  // Tries to create a protobufs::LoopLimiterInfo given a loop header basic
+  // block. Returns true if successful and outputs loop limiter into the |out|
+  // variable. Otherwise, returns false. |out| contains an undefined value when
+  // this function returns false.
+  bool CreateLoopLimiterInfo(
+      opt::IRContext* donor_ir_context, const opt::BasicBlock& loop_header,
+      const std::map<uint32_t, uint32_t>& original_id_to_donated_id,
+      protobufs::LoopLimiterInfo* out);
+
   // Requires that |donated_instructions| represents a prepared version of the
   // instructions of |function_to_donate| (which comes from |donor_ir_context|)
   // ready for donation, and |original_id_to_donated_id| maps ids from
   // |donor_ir_context| to their corresponding ids in the recipient module.
   //
-  // Adds a livesafe version of the function, based on |donated_instructions|,
-  // to the recipient module.
-  void AddLivesafeFunction(
+  // Attempts to add a livesafe version of the function, based on
+  // |donated_instructions|, to the recipient module. Returns true if the
+  // donation was successful, false otherwise.
+  bool MaybeAddLivesafeFunction(
       const opt::Function& function_to_donate, opt::IRContext* donor_ir_context,
       const std::map<uint32_t, uint32_t>& original_id_to_donated_id,
       const std::vector<protobufs::Instruction>& donated_instructions);
@@ -144,12 +154,6 @@ class FuzzerPassDonateModules : public FuzzerPass {
   // array or struct; i.e. it is not an opaque type.
   bool IsBasicType(const opt::Instruction& instruction) const;
 
-  // Returns the ids of all functions in |context| in a topological order in
-  // relation to the call graph of |context|, which is assumed to be recursion-
-  // free.
-  static std::vector<uint32_t> GetFunctionsInCallGraphTopologicalOrder(
-      opt::IRContext* context);
-
   // Functions that supply SPIR-V modules
   std::vector<fuzzerutil::ModuleSupplier> donor_suppliers_;
 };
